//
//  SENTSDK.h
//  SENTSDK
//
//  Created by Emu on 28/07/2017.
//  Copyright © 2018 Sentiance. All rights reserved.
//

@import Foundation;

#import "SENTConfig.h"
#import "SENTPublicDefinitions.h"
#import "SENTVehicleCrashEvent.h"
#import "SENTSDKStatus.h"
#import "SENTUserActivity.h"
#import "SENTTripProcessingTripProfile.h"
#import "SENTTripProcessingTransportSegment.h"
#import "SENTTripProcessingHardEvent.h"
#import "SENTTripProcessingVehicleMode.h"
#import "SENTTripProfilingMode.h"
#import "SENTMetaUsersLinker.h"

extern NSString *const SENTSDKVersion;

@interface SENTSDK : NSObject
/**
 Method to call the shared instance of the SDK.
 */
+ (instancetype)sharedInstance;

/**
 SDK initialisation method. It sets up the modules internally and handles user authentication.
 
 @param config The configuration used to authenticate the user.
 @param success The success block called when init succeeded.
 @param failure The failure block called when init failed. It takes a SENTInitIssue issue indicating
 the reason of the failure.
 @warning Init should only be called once, unless initialisation fails.
 */
- (void)initWithConfig:(SENTConfig *)config
               success:(void (^)(void))success
               failure:(void (^)(SENTInitIssue issue))failure;

/**
 Start the SDK.
 
 @param completion The completion block called when the SDK has started. SENTSDKStatus object
 indicates any issue that occurred when the SDK started.
 @warning start should be called only after the SDK has been successfully initialised.
 */
- (void)start:(void (^)(SENTSDKStatus *status))completion;

/**
 Start the SDK with expiration date.
 
 @param sdkStopDate NSDate when SDK should be stopped. Setting nil, will do regular start,
 without expiration, setting date in the past will not start SDK.
 @param completion The completion block called when the SDK has started. SENTSDKStatus object
 indicates any issue that occurred when the SDK started.
 @warning start should be called only after the SDK has been successfully initialised.
 
 */
- (void)startWithStopDate:(NSDate *)sdkStopDate completion:(void (^)(SENTSDKStatus *status))completion;

/**
 Stop the SDK.
 */
- (void)stop;

/**
 Returns the user ID.
 */
- (NSString *)getUserId;

/**
 Returns the users Access Token if the user is authenticated
 */
- (void)getUserAccessToken:(void (^)(NSString *token))success failure:(void (^)(void))failure;

/**
 Interface method to set metadata for user.
 Metadata is any additional data that enclosing app is willing to set for this user/app combination
 This metadata could contain vital information related to processing wrt to the enclosing app.
 
 @param label Field name for the new entry.
 @param value Value for the new entry.
 
 */
- (void)addUserMetadataField:(NSString *)label value:(NSString *)value;

/**
 Interface method to set metadata for user.
 Metadata is any additional data that enclosing app is willing to set for this user/app combination
 This metadata could contain vital information related to processing wrt to the enclosing app.
 
 @param metadata key value pair data from enclosing app, could have any structure as long as it can
 be contained in a dictionary.
 
 */
- (void)addUserMetadataFields:(NSDictionary *)metadata;

/**
 Method to remove user data field to the sdk metadata (locally as well as on backend).
 
 @param label Field name for the new entry.
 
 */
- (void)removeUserMetadataField:(NSString *)label;

/**
 Start a Trip manually
 */
- (void)startTrip:(NSDictionary *)metadata
transportModeHint:(SENTTransportMode)mode
          success:(void (^)(void))success
          failure:(void (^)(SENTSDKStatus *status))failure;

/**
 Method to add metadata pertaining to the ongoing external trip. All added metadata will be
 aggregated and added to the trip payload. Returns NO if there's no ongoing external trip
 
 @param metadata key value pair data from enclosing app, could have any structure as long as it can
 be contained in a dictionary. The only time we return NO is when a trip is not ongoing, metadata
 contains NSNull objects or nil itself.
 */
- (BOOL)addTripMetadata:(NSDictionary *)metadata;

/**
 Stop a trip.
 */
- (void)stopTrip:(void (^)(void))success failure:(void (^)(SENTSDKStatus *status))failure;

/**
 Set a trip timeout block
 */
- (void)setTripTimeOutListener:(void (^)(void))tripDidTimeOut;

/**
 Checks if vehicle crash detection is supported on the device for the specified trip type.
 The result depends on multiple criteria, such as if vehicle crash detection is enabled for your app, and if
 the necessary sensors are present on the device.
 
 @param tripType the type of trip for which you want to check.
 */
- (BOOL)isVehicleCrashDetectionSupported:(SENTTripType)tripType;


/**
@brief Invokes a dummy vehicle crash event. Use this method to test your vehicle crash detection integration.
@discussion Calling this method will invoke the handler from
@code
- (void)setVehicleCrashHandler:
@endcode
 or
@code
- (void)setCrashListener:
@endcode
@discussion depending on which handler was set last.
@note This method is intended for testing your integration, and not for production use.
*/
- (void)invokeDummyVehicleCrash;

/**
 Set a listener block for vehicle crashes
 */
- (void)setCrashListener:(void (^)(NSDate *date, CLLocation *lastKnownLocation))crashCallback __deprecated_msg("Use setVehicleCrashHandler: instead");

/**
 Sets a handler that is invoked when a vehicle crash is detected.
 */
- (void)setVehicleCrashHandler:(void (^)(SENTVehicleCrashEvent *crashEvent))vehicleCrashHandler;

/**
Set a handler for trip profile detections
*/
- (void)setTripProfileHandler:(void (^) (SENTTripProcessingTripProfile *tripProfile))tripProfileHandler __deprecated;

/**
Set whether the trips should be profiled on-device or on-platform
*/
- (void)setFullTripProfilingEnabled:(BOOL)enabled __deprecated;

/**
Set a speed limit for trip profile detections
*/
- (void)setSpeedLimit:(double)speedLimit __deprecated;

/**
 Interface method to check if trip is in progress or not.
 */
- (BOOL)isTripOngoing:(SENTTripType)tripType;

/**
 This method provides the enclosing app the ability to get SDK detection updates. So when the SDK is
 in a trip or a stationary, it informs the app via a callback, providing some extra information.
 */
- (void)setUserActivityListener:(void (^)(SENTUserActivity *userActivity))onChange;

/**
 This method provides the enclosing app the ability to get SDK current user activity state. It is a
 trip or stationary with some enriched information.
 */
- (SENTUserActivity *)getUserActivity;

/**
 Submits all the pending detections  to backend, without considering any quota related updates and
 returns the status of these submissions via a callback to the enclosing app.
 
 @param success Success block which is to be called when forced submission completes.
 @param failure Failure block which is to be called when forced submission completes.
 
 */
- (void)submitDetections:(void (^)(void))success failure:(void (^)(void))failure;

/**
 Return true if sdk has ever initialized successfully or not
 */
- (BOOL)isInitialised __deprecated_msg("Use -(SENTSDKInitState)getInitState;");

/**
 Return state of initialisation of the SDK: SENTNotInitialized, SENTInitInProgress, SENTInitialized
 */
- (SENTSDKInitState)getInitState;

/**
 Get SDK status information
 */
- (SENTSDKStatus *)getSdkStatus;

/**
 Get SDK version
 */
- (NSString *)getVersion;

/**
 Get SDK WiFi quota limit
 */
- (long)getWifiQuotaLimit;

/**
 Get SDK WiFi quota usage
 */
- (long)getWiFiQuotaUsage;

/**
 Get SDK mobile data quota limit
 */
- (long)getMobileQuotaLimit;

/**
 Get SDK mobile data quota usage
 */
- (long)getMobileQuotaUsage;

/**
 Get SDK disk quota limit
 */
- (long)getDiskQuotaLimit;

/**
 Get SDK disk quota usage
 */
- (long)getDiskQuotaUsage;

/**
 Reset the sdk to initial state, cancel what is in progress and cleanup everyhing
 */
- (void)reset:(void(^)(void))success failure:(void(^)(SENTResetFailureReason reason))failure;

@end
